#!/bin/bash
#
# Utilise le programme latency_flows_tests pour tester les debits/latences entre les machines
# d'un meme cluster en faisant des bissections (moitie de cluster contre moitie de cluster).
# Le script augmente petit a petit le nombre de machines en jeu, afin de pouvoir tracer l'evolution
# et eventuellement reperer un point de saturation (generation d'un fichier plat exploitable par gnuplot).
#
# Copyright (C) 2010 Julien VAUBOURG / Sébastien BADIA
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

if [[ -z "$OAR_NODEFILE" ]]; then
	echo "ERROR : You must have a nodes reservation (oarsub, on a single cluster)." >&2
	exit 1
fi

if [[ -z $1 ]]; then
	echo "Usage: ./flowsSumsBis.sh <step>" >&2
	exit 1
fi

STEP=$1
MACHINES=/tmp/${USER}_machines
PLOTFILE=flowsSumsBis.dat
i=3

head -n1 $OAR_NODEFILE > $MACHINES
sort -u $OAR_NODEFILE >> $MACHINES
NB=$(cat $MACHINES | wc -l)
:> $PLOTFILE

if [[ $NB -lt 3 ]]; then
	echo "ERROR : Tests requires at least 3 nodes (1 for monitoring, at least 2 for exchanges)." >&2
	exit 1
fi

while [[ $i -le $NB ]]; do
	echo "Bisection : calculating the flow sum with $i nodes..."
	mpirun -np $i --mca plm_rsh_agent oarsh -machinefile $MACHINES ./latency_flow_tests -g 2> /dev/null >> $PLOTFILE
	i=$(( i + STEP ))
done

rm $MACHINES
exit 0
